use warnings;
use strict;
use File::Basename;

if ($^O eq "MSWin32")
{
	use lib "C:/Program Files/Actian/Zen/bin";
}
else
{
	use lib "/usr/local/actianzen/lib64";
}

use btrievePerl;

my $min_x = 0;
my $max_x = 255;
my $btrieveFileName = "squaresAndSquareRoots.btr";
my $recordFormat = "(Cvd)<";
my $recordLength = 11;
my $keyFormat = "(C)<";

my $btrieveClient;
my $btrieveFile;
my $btrieveFileAttributes;
my $btrieveIndexAttributes;
my $btrieveKeySegment;
my $integerValue;
my $key;
my $rc;
my $record;
my $x;
my $xSquared;
my $xSquareRoot;

if (@ARGV != 1)
{
	die("Usage: ", basename($0), " uint8_value\n");
}

$integerValue = $ARGV[0];

if (($integerValue < $min_x) or ($integerValue > $max_x))
{
	die("Usage: ", basename($0), " uint8_value\n");
}

$btrieveClient = new btrievePerl::BtrieveClient();
die unless $btrieveClient != 0;

$btrieveFileAttributes = new btrievePerl::BtrieveFileAttributes();
die unless $btrieveFileAttributes != 0;

$rc = $btrieveFileAttributes->SetFixedRecordLength($recordLength);
die "Error: BtrieveFileAttributes->SetFixedRecordLength:" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
   unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$rc = $btrieveClient->FileCreate($btrieveFileAttributes, $btrieveFileName, $btrievePerl::Btrieve::CREATE_MODE_OVERWRITE);
die "Error: BtrieveClient->FileCreate" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
   unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$btrieveFile = new btrievePerl::BtrieveFile();
die unless $btrieveFile != 0;

$rc = $btrieveClient->FileOpen($btrieveFile, $btrieveFileName, "", $btrievePerl::Btrieve::OPEN_MODE_NORMAL);
die "Error: BtrieveClient->FileOpen" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
   unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

# For all the values of x.
for (my $i = $min_x; $i <= $max_x; $i++)
{
        $record = pack($recordFormat, $i, $i * $i, sqrt($i));

        $rc = $btrieveFile->RecordCreate($record);
        die "Error: BtrieveFile->RecordCreate" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
            unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;
}

$btrieveKeySegment = new btrievePerl::BtrieveKeySegment();
die unless $btrieveKeySegment != 0;

$rc = $btrieveKeySegment->SetField(0, 1, $btrievePerl::Btrieve::DATA_TYPE_UNSIGNED_BINARY);
die "Error: BtrieveKeySegment->SetField" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
    unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$btrieveIndexAttributes = new btrievePerl::BtrieveIndexAttributes();
die unless $btrieveIndexAttributes != 0;

$rc = $btrieveIndexAttributes->AddKeySegment($btrieveKeySegment);
die "Error: BtrieveIndexAttributes->AddKeySegment" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
    unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$rc = $btrieveFile->IndexCreate($btrieveIndexAttributes);
die "Error: BtrieveFile->IndexCreate" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
    unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$record = pack($recordFormat, 0, 0, 0);
$key = pack($keyFormat, $integerValue);

$rc = $btrieveFile->RecordRetrieve($btrievePerl::Btrieve::COMPARISON_EQUAL, $btrievePerl::Btrieve::INDEX_1, $key, $record);
die "Error: BtrieveFile->IndexCreate" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($btrieveFile->GetLastStatusCode()) 
    unless $rc == $recordLength;

# Print record.
($x, $xSquared, $xSquareRoot) = unpack($recordFormat, $record);
print "record: (", $x, ", ", $xSquared, ", ", $xSquareRoot, ")\n";

$rc = $btrieveClient->FileClose($btrieveFile);
die "Error: BtrieveClient->FileClose" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
    unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;

$rc = $btrieveClient->FileDelete($btrieveFileName);
die "Error: BtrieveClient->FileDelete" . ":" . $rc . ":" . btrievePerl::Btrieve::StatusCodeToString($rc) 
    unless $rc == $btrievePerl::Btrieve::STATUS_CODE_NO_ERROR;
